package pagerank;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

public class Function {
	class Page {
		String label;
		Double curRank;
		Double lastRank;
		Set<String> inPages;
		Set<String> outPages;

		public Page(String label) {
			super();
			this.label = label;
			curRank = 1.0;
			lastRank = curRank;
			this.inPages = new HashSet<String>();
			this.outPages = new HashSet<String>();
		}

		@Override
		public String toString() {
			// TODO Auto-generated method stub
			return " " + curRank;
		}
	}

	Map<String, Page> pageMap = new HashMap<String, Page>();

	private Double[] iter() {
		Double[] res = new Double[pageMap.size()];
		for (Page p : pageMap.values()) {
			double tmp = 0;
			for (String in : p.inPages) {
				Page inp = pageMap.get(in);
				tmp += inp.lastRank / (double) inp.outPages.size();
			}
			p.curRank = (1 - Comm.damp) / (double) pageMap.size() + Comm.damp * tmp;
		}
		for (Page p : pageMap.values()) {
			p.lastRank = p.curRank;
		}
		@SuppressWarnings("unchecked")
		Map.Entry<String, Page>[] entries = Comm.getSortedHashtableByValue(pageMap, new Comparator<Map.Entry<String, ?>>() {
			@Override
			public int compare(final Entry<String, ?> o1, final Entry<String, ?> o2) {
				// TODO Auto-generated method stub
				return o1.getKey().compareTo(o2.getKey());
			}
		});

		for (int i = 0; i < entries.length; i++) {
			Entry<String, Page> entry = entries[i];
			res[i] = entry.getValue().curRank;
		}

		return res;
	}

	public void readFile(String file) {
		try {
			BufferedReader in = new BufferedReader(new FileReader(file));
			String line;
			while ((line = in.readLine()) != null) {
				String p = line.split("\\s+")[0];
				String po[] = line.split("\\s+")[1].split(",");
				Page cp = null;
				if (pageMap.containsKey(p)) {
					cp = pageMap.get(p);
				} else {
					cp = new Page(p);
				}

				for (String outPage : po) {
					cp.outPages.add(outPage);
				}
				pageMap.put(p, cp);
			}

			for (Page p : pageMap.values()) {
				for (String inPage : p.outPages) {
					Page tp = pageMap.get(inPage);
					tp.inPages.add(p.label);
				}
			}

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void run(String file) {
		readFile(file);

		Double res[] = null;
		for (int i = 0; i < 100; i++) {
			res = iter();
			System.out.println(Arrays.toString(res));
		}
		System.out.println(Arrays.toString(Comm.normMatrix(res)));
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		String rootPath = ClassLoader.getSystemResource("pagerank").getPath();
		Function m = new Function();
		m.run(rootPath + "/page.in");
	}

}
